defmodule Chronoscope.NTSTest do
  use Chronoscope.Case

  alias Chronoscope.NTS
  alias Chronoscope.NTS.DynamicSupervisorMock
  alias Chronoscope.NTS.GenServerMock
  alias Chronoscope.NTS.RegistryMock

  import Chronoscope.NTS
  import Mox

  setup :verify_on_exit!

  describe "Chronoscope.NTS.healthy?()" do
    test "is healthy" do
      assert healthy?() == true
    end
  end

  describe "Chronoscope.NTS.list()" do
    test "shows empty client list" do
      DynamicSupervisorMock
      |> expect(:which_children, fn _ -> [] end)

      assert list() == []
    end

    test "lists all children" do
      DynamicSupervisorMock
      |> expect(:which_children, fn _ -> [{1, 2, 3, 4}, {5, 6, 7, 8}] end)

      GenServerMock
      |> expect(:call, fn 2, :list -> :one end)
      |> expect(:call, fn 6, :list -> :two end)

      assert list() == [:one, :two]
    end
  end

  describe "Chronoscope.NTS.remove()" do
    test "does nothing if the client doesn't exist" do
      RegistryMock
      |> expect(:lookup, fn _, _ -> [] end)

      assert remove("localhost", 1111) == {:error, :not_found}
    end

    test "removes a client" do
      RegistryMock
      |> expect(:lookup, fn _, _ -> [{1, 2}] end)

      GenServerMock
      |> expect(:call, fn 1, :terminate -> :terminating end)

      assert remove("localhost", 1111) == {:ok, :terminating}
    end
  end

  describe "Chronoscope.NTS.key_establishment()" do
    test "creates a new client" do
      RegistryMock
      |> expect(:lookup, fn _, _ -> [] end)

      DynamicSupervisorMock
      |> expect(:start_child, fn NTS.DynamicSupervisor,
                                 {NTS.Client,
                                  [
                                    server: %{host: "localhost", port: 1111},
                                    name: {:via, RegistryMock, {NTS.Registry, "localhost:1111"}}
                                  ]} ->
        {:ok, 1}
      end)

      GenServerMock
      |> expect(:call, fn 1, :key_establishment -> :result end)

      assert key_establishment("localhost", 1111) == :result
    end

    test "reuses an existing client" do
      RegistryMock
      |> expect(:lookup, fn _, _ -> [{1, 2}] end)

      GenServerMock
      |> expect(:call, fn 1, :key_establishment -> :result end)

      assert key_establishment("localhost", 1111) == :result
    end
  end
end
